class Joint {
  int x = -1;
  int y = -1;
  color c;
  boolean selected = false;
  String name;
  int id;

  Joint( int _id) {
    id = _id;
  }


  Joint( int _id, color _c) {
    c=_c;
    id = _id;
  }

  Joint(String _name, color _c, int _id) {
    name = _name;
    c=_c;
    id = _id;
  }

  void setXY(int _x, int _y) {
    x =_x;
    y = _y;
  }
}

void addJointList() {
  joints.add( new Joint("Green", #7ebf29, 0) );
  joints.add( new Joint("Teal", #2bffce, 1) );
  joints.add( new Joint("Orange", #ff9f32, 2) );
  joints.add( new Joint("Purple", #cf6fff, 3) );
  joints.add( new Joint("Red", #e85107, 4) );
  joints.add( new Joint("Yellow", #ffe04f, 5) );
  joints.add( new Joint("Blue", #6259ff, 6) );
  joints.add( new Joint("Grey", #919191, 7) );
  joints.add( new Joint("Black", #000000, 8) );
}

class JointList extends Controller<JointList> {
  ArrayList<Joint> setImages; 
  float pos, npos;
  int itemHeight = 50;
  int scrollerLength = 40;
  PGraphics menu;
  boolean updateMenu;
  int totalHeight=0;
  int w, h;
  int btSize = 50;

  JointList(ControlP5 c, String theName, int theWidth, int theHeight, ArrayList<Joint> _setImages) {
    super( c, theName, 0, 0, theWidth, theHeight );
    c.register( this );
    w = theWidth;
    setImages = _setImages;
    updateHeight(theWidth);
    menu = createGraphics(getWidth(), getHeight() );
    setView(new ControllerView<JointList>() {
      public void display(PGraphics pg, JointList t ) {
        if (updateMenu) {
          updateMenu();
        }
        if (inside() ) {
          menu.beginDraw();
          //int len = -(itemHeight * items.size()) + getHeight();
          //int ty = int(map(pos, len, 0, getHeight() - scrollerLength - 2, 2 ) );
          // menu.rect(getWidth()-4, ty, 4, scrollerLength );
          menu.endDraw();
        }
        pg.image(menu, 0, 0);
      }
    }
    );
    updateMenu();
  }

  void updateMenu() {
    int len = -(totalHeight) + getHeight();
    npos = constrain(npos, len, 0);
    if (totalHeight < getHeight() ) {
      pos = 0;
    } else {
      pos += (npos - pos) * 0.1;
    }
    if (inside() && setImages.size() > 0) {
      int len2 = itemHeight * setImages.size();
      int index = int( map( getPointer().y() - pos, 0, len2, 0, setImages.size() ) ) ;
      drawNormal();
      if (index < setImages.size() ) {
        drawOver(index);
      }
    } else {
      drawNormal();
    }
    updateMenu = abs(npos-pos)>0.01 ? true:false;
  }

  void drawOver(int index) {
    int mX = getPointer().x();
    menu.beginDraw();
    menu.pushMatrix();
    menu.translate( 0, itemHeight *index );
    menu.pushMatrix();
    menu.textFont(iconFont);
    menu.textSize(21);
    menu.textAlign(CENTER, CENTER);

    menu.popMatrix();
    menu.popMatrix();
    menu.endDraw();
  }

  void drawNormal() {
    menu.beginDraw();
    menu.pushMatrix();
    menu.translate( 0, pos );
    menu.pushMatrix();
    menu.noStroke();
    menu.background(240);
    menu.textAlign(LEFT);
    for (Joint a : setImages) {
      if (a.selected) {
        menu.fill(a.c);
        menu.rect(0, 0, w, itemHeight-1);
        menu.textFont(iconFont);
        menu.textAlign(CENTER, CENTER);
        menu.fill(255);
        menu.textSize(21);
        menu.text("\ue836", btSize/2, btSize/2-2);
        menu.textAlign(LEFT);
        menu.textFont(font);
        menu.textSize(16);
        menu.fill(255);
        menu.text(a.name, 48, 28);
      } else {
        menu.fill(255);
        menu.rect(0, 0, w, itemHeight-1);
        //menu.textAlign(CENTER);
        menu.textFont(iconFont);
        menu.textAlign(CENTER, CENTER);
        menu.fill(a.c);
        menu.textSize(21);
        menu.text("\ue836", btSize/2, btSize/2-2);
        menu.textAlign(LEFT);
        menu.textFont(font);
        menu.textSize(16);
        menu.fill(0);
        menu.text(a.name, 48, 29);
      }
      menu.translate(0, itemHeight);
    }
    menu.popMatrix();
    menu.popMatrix();
    menu.endDraw();
  }

  public void onMove() {
    updateMenu = true;
  }

  public void onEnter() {
    updateMenu = true;
  }

  void onLeave() {
    updateMenu = true;
  }

  public void onRelease() {
    int len = itemHeight * setImages.size();
    int index = int( map( getPointer().y() - pos, 0, len, 0, setImages.size() ) ) ;
    if (index < setImages.size() ) {
      Joint s = setImages.get(index);
      for (Joint a : setImages) {
        a.selected=false;
        s.selected=true;
      }
      GLOBAL_joint = index;
      updateMenu = true;
    }
    println(GLOBAL_joint);
  }

  public void onDrag() {
    npos += getPointer().dy() * 2;
    updateMenu = true;
  } 

  public void onScroll(int n) {
    npos += ( n * 4 );
    updateMenu = true;
  }

  void updateHeight(int w) {
    totalHeight = itemHeight * setImages.size();
  }
}
