PrintWriter output;

void loadButton() {
  selectInput("Select a file to process:", "loadSelected");
}

void saveButton() {
  selectOutput("Select a file to process:", "saveSelected");
}

void saveSelected(File selection) {
  if (selection == null) {
    println("Window was closed or the user hit cancel.");
  } else {
    output = createWriter(selection.getAbsolutePath());
    output.print("SETS"+'\t');
    for (int i = 0; i<imageSets.size(); i++) {
      ImageSet set = imageSets.get(i);
      output.print(set.absFolderName+'\t');
    }
    output.print('\n');
    output.print("DORDER"+'\t');
    for (int i = 0; i<dOrder.size(); i++) {
      int d = dOrder.get(i);
      output.print(String.valueOf(d)+'\t');
    }
    output.print('\n');
    output.print("JOINTS"+'\t');
    for (int i = 0; i<imageSets.size(); i++) {
      ImageSet set = imageSets.get(i);
      for (int j=0; j<set.aImages.size(); j++) {
        AnchorImage ai = set.aImages.get(j);
        for (int k=0; k<ai.joints.size(); k++) {
          Joint jnt = ai.joints.get(k);
          // PATTERN = SET  ANCHORIMAGE  JOINT.ID JOINT.X JOINT.Y
          output.print(String.valueOf(i)+'\t'+String.valueOf(j)+'\t'+String.valueOf(jnt.id)+'\t'+String.valueOf(jnt.x)+'\t'+String.valueOf(jnt.y)+'\t');
        }
      }
    }
    output.flush(); // Writes the remaining data to the file
    output.close(); // Finishes the file
  }
}


void loadSelected(File selection) {
  if (selection == null) {
    println("Window was closed or the user hit cancel.");
  } else {
    ArrayList<String> temp = new ArrayList<String>();
    // Open the file from the createWriter() example
    BufferedReader reader = createReader(selection.getAbsolutePath());
    String line = null;
    try {
      clearAll();
      while ((line = reader.readLine()) != null) {
        String[] pieces = split(line, TAB);
        String id = pieces[0];
        switch (id) {
        case "SETS":
          for ( int i=1; i<pieces.length-1; i++ ) {
            temp.add(pieces[i]);
          }
          for ( String s : temp ) {
            loatImageSets(s);
          }
          setBrowser.updateHeight();
          break;
        case "DORDER":
          dOrder.clear();
          for ( int i=1; i<pieces.length-1; i++ ) {
            dOrder.add(int(pieces[i]));
          }
          updateDrawOrder();
          break;
        case "JOINTS":
          for ( int i=1; i<pieces.length-1; i+=5 ) {
            ImageSet set = imageSets.get( int(pieces[i]) );
            AnchorImage img = set.aImages.get( int(pieces[i+1]) );
            Joint j = new Joint( int(pieces[i+2]), joints.get( int(pieces[i+2]) ).c );
            j.setXY( int(pieces[i+3]), int(pieces[i+4]) );
            img.joints.add(j);
          }
          break;
        }
      }
      reader.close();
      printArray( temp );
    } 
    catch (IOException e) {
      e.printStackTrace();
    }
  }
}


void saveCreate() {
  output = createWriter("save.txt");
  output.print("SETS"+'\t');
  for (int i = 0; i<imageSets.size(); i++) {
    ImageSet set = imageSets.get(i);
    output.print(set.absFolderName+'\t');
  }
  output.print('\n');
  output.print("DORDER"+'\t');
  for (int i = 0; i<dOrder.size(); i++) {
    int d = dOrder.get(i);
    output.print(String.valueOf(d)+'\t');
  }
  output.print('\n');
  output.print("JOINTS"+'\t');
  for (int i = 0; i<imageSets.size(); i++) {
    ImageSet set = imageSets.get(i);
    for (int j=0; j<set.aImages.size(); j++) {
      AnchorImage ai = set.aImages.get(j);
      for (int k=0; k<ai.joints.size(); k++) {
        Joint jnt = ai.joints.get(k);
        // PATTERN = SET  ANCHORIMAGE  JOINT.ID JOINT.X JOINT.Y
        output.print(String.valueOf(i)+'\t'+String.valueOf(j)+'\t'+String.valueOf(jnt.id)+'\t'+String.valueOf(jnt.x)+'\t'+String.valueOf(jnt.y)+'\t');
      }
    }
  }
  output.flush(); // Writes the remaining data to the file
  output.close(); // Finishes the file
}

void loadSave() {
  ArrayList<String> temp = new ArrayList<String>();
  // Open the file from the createWriter() example
  BufferedReader reader = createReader("save.txt");
  String line = null;
  try {
    clearAll();
    while ((line = reader.readLine()) != null) {
      String[] pieces = split(line, TAB);
      String id = pieces[0];
      switch (id) {
      case "SETS":
        for ( int i=1; i<pieces.length-1; i++ ) {
          temp.add(pieces[i]);
        }
        for ( String s : temp ) {
          loatImageSets(s);
        }
        setBrowser.updateHeight();
        break;
      case "DORDER":
        dOrder.clear();
        for ( int i=1; i<pieces.length-1; i++ ) {
          dOrder.add(int(pieces[i]));
        }
        updateDrawOrder();
        break;
      case "JOINTS":
        for ( int i=1; i<pieces.length-1; i+=5 ) {
          ImageSet set = imageSets.get( int(pieces[i]) );
          AnchorImage img = set.aImages.get( int(pieces[i+1]) );
          Joint j = new Joint( int(pieces[i+2]), joints.get( int(pieces[i+2]) ).c );
          j.setXY( int(pieces[i+3]), int(pieces[i+4]) );
          img.joints.add(j);
        }
        break;
      }
    }
    reader.close();
    printArray( temp );
  } 
  catch (IOException e) {
    e.printStackTrace();
  }
} 

void clearAll() {
  browser.deselectAll();
  setBrowser.deselectAll();
  imageSets.clear();
  //joints.clear();
  dOrder.clear();
}

void loatImageSets(String folder) {
  //String folder = selection.getAbsolutePath();
  File imgsdir = new File(folder);
  String[] extensions = new String[] { "tga", "jpg", "png" };
  List<File> files = (List<File>) FileUtils.listFiles(imgsdir, extensions, false);
  if (files.size() >0 ) {
    ImageSet is = new ImageSet( files, imgsdir.getName() );
    is.absFolderName = folder;
    imageSets.add( is );
    dOrder.add(imageSets.size()-1);
    updateDrawOrder();
  }
}
